//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  Native American Flute Fingering Diagrams
//  Copyright (C) 2016 Jeffrey Edwards
//
//  created from / based on:
//  Note Names Plugin
//  Copyright (C) 2012 Werner Schweer
//  Copyright (C) 2013, 2014 Joachim Schmitz
//  Copyright (C) 2014 Jörn Eichler
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//
//  This plugin requires the installation of the NAF 6 hole flute font "NAFTracks Six Hole"
//  created by Clint Goss and available on Flutopedia.com
//
//  Thanks to Clint Goss and the makers of the Note Names Plugin
//  for making this possible
//=============================================================================

import QtQuick 2.0
import MuseScore 1.0

MuseScore {
   version: "3.0"
   description: "This plugin adds Native American Flute Fingering Diagrams"
   menuPath: "Plugins.Native American Flute"

//=============================================================================
   function addFINGERING (cursor, note, posX, posY, isGrace) {
      //isGrace=1 > gracenote, isGrace=0 > note
      var text = newElement(Element.STAFF_TEXT);
      switch (isGrace){
         case 0: text.text = "<font size=\"50\"/>"; break;
         case 1: text.text = "<font size=\"40\"/>"; break;
      }// end for switch

      text.text = text.text + "<font face=\"NAFTracks Six Hole\"/>"

      //if the fingerings require half-holes or are in second octave then make them red
      //comment out this switch if you do not want this feature
      switch (note.pitch) {
         case  67: text.color = "red"; break;
         case  68: text.color = "red"; break;
         case  79: text.color = "red"; break;
         case  80: text.color = "red"; break;
         case  81: text.color = "red"; break;
         case  82: text.color = "red"; break;
         case  83: text.color = "red"; break;
      } // end for switch

      //determine the fingering for the note
      switch (note.pitch) {
         case  66: text.text = text.text + "1";      break; //F# XXX_XXX 1
         case  67: text.text = text.text + "!";      break; //G  XXX_XXQ !
         case  68: text.text = text.text + "2";      break; //G# XXX_XXH 2
         case  69: text.text = text.text + "3";      break; //A  XXX_XXO 3
         case  70: text.text = text.text + "#";      break; //A# XXX_XOX #
         case  71: text.text = text.text + "4";      break; //B  XXX_XOO 4
         case  72: text.text = text.text + "$";      break; //C  XXX_OXO $
         case  73: text.text = text.text + "5";      break; //C# XXX_OOO 5
         case  74: text.text = text.text + "6";      break; //D  XXO_XOO 6
         case  75: text.text = text.text + "^";      break; //D# XXO_OOO ^  Alt+094
         case  76: text.text = text.text + "7";      break; //E  XOX_OOO 7
         case  77: text.text = text.text + "&amp;";  break; //F  OXX_OOO &  Alt+038  \u0026
         case  78: text.text = text.text + "8";      break; //F# OOX_OOO 8
         case  79: text.text = text.text + "*";      break; //G  OXX_XXX *
         case  80: text.text = text.text + "\u0088"; break; //G# HXX_XXH ˆ  Alt+0136
         case  81: text.text = text.text + "\u008a"; break; //A  HXX_XXO Š  Alt+0138
         case  82: text.text = text.text + "\u008c"; break; //A# HXX_XHO Œ  Alt+0140 \u008C
         case  83: text.text = text.text + "\u008e"; break; //B  HXX_XOO Ž  Alt+0142
      } // end for switch

      text.pos.x=posX
      text.pos.y=posY
      cursor.add(text);
   }// end function addFINGERING

//=============================================================================

   onRun: {
      if (typeof curScore === 'undefined')
         Qt.quit();
      var cursor = curScore.newCursor();
      var startStaff;
      var endStaff;
      var endTick;
      var fullScore = false;
      cursor.rewind(1);
      if (!cursor.segment) { // no selection
         fullScore = true;
         startStaff = 0; // start with 1st staff
         endStaff  = curScore.nstaves - 1; // and end with last
      } else {
         startStaff = cursor.staffIdx;
         cursor.rewind(2);
         if (cursor.tick == 0) {
            // this happens when the selection includes
            // the last measure of the score.
            // rewind(2) goes behind the last segment (where
            // there's none) and sets tick=0
            endTick = curScore.lastSegment.tick + 1;
         } else {
            endTick = cursor.tick;
         }
         endStaff   = cursor.staffIdx;
      }

      for (var staff = startStaff; staff <= endStaff; staff++) {
         for (var voice = 0; voice < 4; voice++) {
            cursor.rewind(1); // beginning of selection
            cursor.voice    = voice;
            cursor.staffIdx = staff;

            if (fullScore)  // no selection
               cursor.rewind(0); // beginning of score

            while (cursor.segment && (fullScore || cursor.tick < endTick)) {
               if (cursor.element && cursor.element.type == Element.CHORD) {

                  var notes = cursor.element.notes;
                  addFINGERING(cursor, notes[0], 0, 9, 0);

                  // iterate through all grace chords
                  var graceChords = cursor.element.graceNotes;
                  for (var i = 0; i < graceChords.length; i++) {
                     var notes = graceChords[i].notes;
                     // there seems to be no way of knowing the exact horizontal pos.
                     // of a grace note, so we have to guess:
                     // we need a better way of determining the Xposition of the gracenote
                     // until then adjust position manually
                     var posX = 2.5 * (graceChords.length - i);
                     addFINGERING(cursor, notes[0], posX, 9, 1);				
                  }

               } // end if CHORD

               cursor.next();
            } // end while segment

         } // end for voice
      } // end for staff
      Qt.quit();
   } // end onRun
} //end MuseScore
